
# coding: utf-8

# In[1]:


# Created on 04/27
# Mohammed Tawashi - UCSD

# Processing strictly hashtags associated with media files
# Some sources: Naveed's topic modeling NB

# 7 days data (12/3 - 12/9)


# In[2]:


import collections
import configparser
import gensim
import itertools 
import matplotlib.pyplot as plt
import networkx as nx
import nltk
import numpy as np
import os
import pandas as pd
import pickle 
import psycopg2
import pyLDAvis
import re
import warnings
import seaborn as sns
import matplotlib
import random
import community
import networkx as nx
import time
import networkx.algorithms.community as nxcom
from matplotlib import pyplot as plt
from numpy import random as nprand
from itertools import combinations
from sklearn.model_selection import GridSearchCV
from sklearn.metrics.pairwise import cosine_similarity
from collections import OrderedDict
from gensim import corpora, models
from gensim.parsing.preprocessing import STOPWORDS
from gensim.summarization import mz_keywords
from gensim.utils import simple_preprocess
from mlxtend.preprocessing import TransactionEncoder
from nltk import bigrams
from nltk.corpus import stopwords
from nltk.sentiment.vader import SentimentIntensityAnalyzer
from nltk.stem import WordNetLemmatizer, SnowballStemmer
from nltk.stem.porter import *
from pandas import DataFrame
from pyLDAvis import sklearn as sklearn_lda
from sklearn.datasets import fetch_20newsgroups
from sklearn.decomposition import LatentDirichletAllocation as LDA
from sklearn.decomposition import NMF, LatentDirichletAllocation
from sklearn.decomposition import NMF, non_negative_factorization
from sklearn.feature_extraction.text import CountVectorizer
from sklearn.feature_extraction.text import TfidfVectorizer, CountVectorizer
from textblob import TextBlob
from time import sleep
import operator
from collections import defaultdict
get_ipython().magic('matplotlib inline')
nltk.download('stopwords')
nltk.downloader.download('vader_lexicon')
warnings.simplefilter("ignore", DeprecationWarning)


# ## Topic modeling functions

# In[3]:


# NMF model function

def build_nmf(data):
    n_samples = 2000
    n_features = 1000
    n_components = 10 #Number of topics
    n_words = 20 #Number of top words to choose from each topic
    data_samples=data
    # Use tf-idf features for NMF.
    tfidf_vectorizer = TfidfVectorizer(max_df=0.95, min_df=2,
                                   max_features=n_features,
                                   stop_words=set(stopwords.words('english')))
    tfidf = tfidf_vectorizer.fit_transform(data_samples)
    nmf = NMF(n_components=n_components, random_state=1,
          beta_loss='kullback-leibler', solver='mu', max_iter=1000, alpha=.1,
          l1_ratio=.5).fit(tfidf)
    
    keywords = np.array(tfidf_vectorizer.get_feature_names())
    topic_keywords = []
    for topic_weights in nmf.components_:
        top_keyword_locs = (-topic_weights).argsort()[:n_words]
        topic_keywords.append(keywords.take(top_keyword_locs))
    df_topic_keywords = pd.DataFrame(topic_keywords)
    df_topic_keywords.columns = ['Word '+str(i) for i in range(df_topic_keywords.shape[1])]
    df_topic_keywords.index = ['Topic '+str(i) for i in range(df_topic_keywords.shape[0])]

    return tfidf_vectorizer,nmf,df_topic_keywords


# In[4]:


# Define function to predict topic for a given new text

def predict_topic(tfidf_vectorizer,nmf,df_topic_keywords,text):
    
    # Step 1: Vectorize transform
    newtext = tfidf_vectorizer.transform(text)
    # Step 2: NMF Transform
    topic_probability_scores = nmf.transform(newtext)

    topic_name = df_topic_keywords.iloc[np.argmax(topic_probability_scores), :].name
    topic = df_topic_keywords.iloc[np.argmax(topic_probability_scores), :].values.tolist()
    return topic_name,topic


# ## Hashtags alone

# In[5]:


# Hashtags aggregated text
conn = None
sql="SELECT * from hashtags_agg order by uniq_users DESC;"
try:
    config = configparser.ConfigParser()
    config.read('config.ini')
    conn = psycopg2.connect(**config['postgres']) 
    cur = conn.cursor()
    cur.execute(sql)
    hashtags_agg = DataFrame(cur.fetchall())
    cur.close()
except (Exception, psycopg2.DatabaseError) as error:
    print(error)
finally:
    if conn is not None:
        conn.close()
hashtags_agg.rename(columns={0:'hashtag',1:'uniq_users',2:'uniq_tweets',3:'ht_text'}, inplace=True)


# In[6]:


hashtags_agg.head()


# In[7]:


# train NMF model on hashtags agg text
tfidf_vectorizer,nmf,df_topic_keywords=build_nmf(hashtags_agg['ht_text'])


# In[54]:


df_topic_keywords


# In[8]:


# step1_df - copy
step1_df = hashtags_agg.copy()


# In[9]:


# Generate topics for each hashtag using predict function. Seperate into 2 different columns
step1_df['topic_name'],step1_df['topic']  = step1_df['ht_text'].apply(lambda text: predict_topic(tfidf_vectorizer,nmf,df_topic_keywords,[text])).str


# In[10]:


step1_df.head()


# In[21]:


step1_df.to_pickle('hashtags_agg')


# ## hashtag_cooccurence_w_media

# In[11]:


# Hashtags aggregated text
conn = None
sql="SELECT * from hashtag_cooccurence_w_media order by uniq_users DESC;"
try:
    config = configparser.ConfigParser()
    config.read('config.ini')
    conn = psycopg2.connect(**config['postgres']) 
    cur = conn.cursor()
    cur.execute(sql)
    hashtag_cooc_w_media = DataFrame(cur.fetchall())
    cur.close()
except (Exception, psycopg2.DatabaseError) as error:
    print(error)
finally:
    if conn is not None:
        conn.close()
hashtag_cooc_w_media.rename(columns={0:'ht1',1:'ht2',2:'uniq_users', 3:'uniq_tweets', 4:'ht_text'}, inplace=True)


# In[12]:


hashtag_cooc_w_media.head()


# In[13]:


# step1_df - copy
step2_df = hashtag_cooc_w_media.copy()


# In[14]:


# Generate topics for each hashtag using predict function. Seperate into 2 different columns
step2_df['topic_name'],step2_df['topic']  = step2_df['ht_text'].apply(lambda text: predict_topic(tfidf_vectorizer,nmf,df_topic_keywords,[text])).str


# In[15]:


step2_df.head()


# In[24]:


step2_df.to_pickle('hashtag_cooc_w_media')


# In[31]:


step2_df['ht_pair'] = step2_df['ht1'].astype(str) +' , '+ step2_df['ht2']


# In[32]:


step2_df[step2_df['uniq_users']>100]


# In[44]:


# Hashtags visualization
B = step2_df[step2_df['uniq_users']>100]

plt.figure(figsize=(16, 10))
B.plot.bar(x='ht_pair', y='uniq_users',ax = plt.gca())

#B.plot.bar()
plt.xlabel('Hashtags Pair', fontsize=16)
plt.ylabel('Co-occurrence times (# users)', fontsize=16)
plt.title('Hashtags Co-occurrences for Tweets associated with Media', fontsize=20)
plt.grid()
plt.show()


# In[121]:


# user_concentration_factor calculation
step2_df['user_concentration_factor'] = step2_df['uniq_tweets']/step2_df['uniq_users']
step2_df.head()


# ## hashtag_cooccurence with no media

# In[16]:


# hashtag_cooc_wo_media
conn = None
sql="SELECT * from hashtag_cooccurence_wo_media order by uniq_users DESC;"
try:
    config = configparser.ConfigParser()
    config.read('config.ini')
    conn = psycopg2.connect(**config['postgres']) 
    cur = conn.cursor()
    cur.execute(sql)
    hashtag_cooc_wo_media = DataFrame(cur.fetchall())
    cur.close()
except (Exception, psycopg2.DatabaseError) as error:
    print(error)
finally:
    if conn is not None:
        conn.close()
hashtag_cooc_wo_media.rename(columns={0:'ht1',1:'ht2',2:'uniq_users', 3:'uniq_tweets', 4:'ht_text'}, inplace=True)


# In[17]:


hashtag_cooc_wo_media.head()


# In[18]:


# step1_df - copy
step3_df = hashtag_cooc_wo_media.copy()


# In[19]:


# Generate topics for each hashtag using predict function. Seperate into 2 different columns
step3_df['topic_name'],step3_df['topic']  = step3_df['ht_text'].apply(lambda text: predict_topic(tfidf_vectorizer,nmf,df_topic_keywords,[text])).str


# In[20]:


step3_df.head()


# In[25]:


step3_df.to_pickle('hashtag_cooc_wo_media')


# In[45]:


step3_df['ht_pair'] = step3_df['ht1'].astype(str) +' , '+ step3_df['ht2']


# In[46]:


# Hashtags visualization
B = step3_df[step3_df['uniq_users']>1000]

plt.figure(figsize=(16, 10))
B.plot.bar(x='ht_pair', y='uniq_users',ax = plt.gca())

#B.plot.bar()
plt.xlabel('Hashtags Pair', fontsize=16)
plt.ylabel('Co-occurrence times (# users)', fontsize=16)
plt.title('Hashtags Co-occurrences for Tweets NOT associated with Media', fontsize=20)
plt.grid()
plt.show()


# In[119]:


# user_concentration_factor calculation
step3_df['user_concentration_factor'] = step3_df['uniq_tweets']/step3_df['uniq_users']
step3_df.head()


# ## Intersection and Topic analysis

# In[122]:


# Media
step21_df = step2_df.set_index('ht_pair')
step21_df.head()


# In[123]:


# No media
step31_df = step3_df.set_index('ht_pair')
step31_df.head()


# In[124]:


# Intersection hashtag pairs (based on index)
hash_inter = step21_df.index.intersection(step31_df.index)


# In[125]:


len(hash_inter)


# In[126]:


hash_inter


# In[127]:


# Media and no media intersected hashtags
hashs_weights_Media_inter = step21_df.loc[hash_inter]
hashs_weights_NoMedia_inter = step31_df.loc[hash_inter]


# In[134]:


hashs_weights_Media_inter.head()


# In[91]:


# hashs_weights_Media_inter
#B = hashs_weights_Media_inter[hashs_weights_Media_inter['uniq_users']>100]['uniq_users']
B = hashs_weights_Media_inter['uniq_users']

plt.figure(figsize=(16, 10))
B.plot.bar(x='ht_pair', y='uniq_users',ax = plt.gca())
plt.xlabel('Hashtags Pair', fontsize=16)
plt.ylabel('Co-occurrence times (# users)', fontsize=16)
plt.title('Hashtags Co-occurrences for Tweets associated with Media', fontsize=20)
#plt.grid()
plt.show()


# In[131]:


# hashs_weights_Media_inter
#B = hashs_weights_NoMedia_inter[hashs_weights_NoMedia_inter['uniq_users']>1000]['uniq_users']
#B = hashs_weights_NoMedia_inter['uniq_users']

#plt.figure(figsize=(16, 10))
#B.plot.bar(x='ht_pair', y='uniq_users',ax = plt.gca())
#plt.xlabel('Hashtags Pair', fontsize=16)
#plt.ylabel('Co-occurrence times (# users)', fontsize=16)
#plt.title('Hashtags Co-occurrences for Tweets NOT associated with Media', fontsize=20)
#plt.grid()
#plt.show()


# In[133]:


step21_df.uniq_users


# In[132]:


step31_df.uniq_users


# In[169]:


# Convert both pd.series into 1 df to keep them aligned

# Media pd.series
Inter_df = pd.DataFrame(hashs_weights_Media_inter.uniq_users)
Inter_df.rename(columns={'uniq_users': 'Weight_Media'}, inplace=True)

# NoMedia pd.series
Inter_df['Weight_NoMedia'] = hashs_weights_NoMedia_inter.uniq_users


# In[170]:


Inter_df.head(10)


# In[164]:


# Filter based on media weights
final_df = Inter_df[Inter_df['Weight_Media'] > 80]


# In[139]:


# Hashtags side by side visualization

final_df.plot.bar(figsize=(16, 10))
plt.xlabel('Hashtags Pair', fontsize=16)
plt.ylabel('Co-occurrence times (# users)', fontsize=16)
plt.title('Hashtags Co-occurrences comparison for Tweets associated/NOT with Media', fontsize=20)
plt.grid()
plt.show()


# In[172]:


# Filter based on media weights
final_df = Inter_df[Inter_df['Weight_Media'] > 50]

# Hashtags side by side visualization

final_df.plot.bar(figsize=(16, 10))
plt.xlabel('Hashtags Pair', fontsize=16)
plt.ylabel('Co-occurrence times (# users)', fontsize=16)
plt.title('Hashtags Co-occurrences comparison for Tweets associated/NOT with Media', fontsize=20)
plt.grid()
plt.show()


# #### Case1 - KAG , MAGA

# In[159]:


# Media co-occurence
hashs_weights_Media_inter[hashs_weights_Media_inter.index =='KAG , MAGA']


# In[160]:


# No Media Co-occurrence
hashs_weights_NoMedia_inter[hashs_weights_NoMedia_inter.index =='KAG , MAGA']


# In[161]:


# Hashtags alone
step1_df[step1_df['hashtag'] == 'KAG']


# In[162]:


# Hashtags alone
step1_df[step1_df['hashtag'] == 'MAGA']


# #### Case2 - UFCDC , UFCWashnigton

# In[173]:


# Media co-occurence
hashs_weights_Media_inter[hashs_weights_Media_inter.index =='UFCDC , UFCWashington']


# In[174]:


# No Media Co-occurrence
hashs_weights_NoMedia_inter[hashs_weights_NoMedia_inter.index =='UFCDC , UFCWashington']


# In[175]:


# Hashtags alone
step1_df[step1_df['hashtag'] == 'UFCDC']


# In[177]:


# Hashtags alone
step1_df[step1_df['hashtag'] == 'UFCWashington']


# ## Hashtags User concentration analysis

# In[135]:


hashs_weights_Media_inter.head()


# In[144]:


# Convert both pd.series into 1 df to keep them aligned

# Media pd.series
Inter1_df = pd.DataFrame(hashs_weights_Media_inter.user_concentration_factor)
Inter1_df.rename(columns={'user_concentration_factor': 'user_concentration_factor_Media'}, inplace=True)

# NoMedia pd.series
Inter1_df['user_concentration_factor_NoMedia'] = hashs_weights_NoMedia_inter.user_concentration_factor


# In[145]:


Inter1_df.head()


# In[147]:


Inter1_df['Diff'] = Inter1_df['user_concentration_factor_Media'] - Inter1_df['user_concentration_factor_NoMedia']
Inter1_df.head()


# In[152]:


### Case1 - hashtag hijaking using media files
Inter1_df[Inter1_df['Diff'] > 20]


# In[151]:


### Case2 - possible media hashtag hijaking 
Inter1_df[Inter1_df['Diff'] < -20]


# In[178]:


hashs_weights_Media_inter[hashs_weights_Media_inter.index =='gif , gifs']


# In[179]:


hashs_weights_Media_inter[hashs_weights_Media_inter.index =='antitrump , donaldtrump']


# In[180]:


hashs_weights_NoMedia_inter[hashs_weights_NoMedia_inter.index =='antitrump , donaldtrump']

