from pyspark.sql import SparkSession
from pyspark.sql.functions import split, explode, get_json_object
from pyspark.sql.types import ArrayType, StringType
from database import save
import json
import hashlib
import datetime

CSV_FILE = "/Users/vamsi/Downloads/200k_tweets.csv"

def __save_media__(df):

    tweet_media = \
    df.select(['id', get_json_object(df.media, '$.media').alias('media_list')]) \
        .rdd \
        .flatMap(lambda row : [(row['id'], w) for w in json.loads(row['media_list'])]) \
        .map(lambda r : (r[0], r[1]['type'], r[1]['video'], 'img' + hashlib.md5(r[1]['video'].encode()).hexdigest())) \
        .collect()
    #print(tweet_media)
    print("Saved Media")
    save('media', tweet_media)

def __save_users__(df):
    users = df.select("userid") \
                .distinct() \
                .rdd \
                .map(lambda row: (row[0], None, None)) \
                .collect()
    print("Saved Users")
    save('twitter_user', users)


def __save_tweets__(df):
    columns = ["id", 
                "userid", 
                "created_at", 
                "text", 
                "reply_count", 
                "retweet_count", 
                "lang"]

    tweets = df.select(columns) \
                    .rdd \
                    .map(lambda row : (row[0],row[1],row[2],row[3],None, None, row[4],row[5],row[6])) \
                    .collect()
    print("Saved Tweets")
    save('tweet', tweets)

def save_image_features(image_feat_list):
    save('image_features', image_feat_list)

def __save_hashtags__(df):
    hashtags = \
    df.filter(df['hashtag'].isNotNull()) \
        .select(["id", "hashtag"]) \
        .withColumn('hashtag', explode(split('hashtag', ','))) \
        .rdd \
        .map(lambda row : (row[0], row[1].replace('{','').replace('}',''))) \
        .distinct() \
        .collect()
    print("Saved Hashtags")
    save('hashtags', hashtags)

def load_file(spark_session):
        df = spark_session \
            .read \
            .format('com.databricks.spark.csv').\
                                options(header='true', \
                                inferschema='true',
                                escape='"',
                                multiline='true')\
            .load(CSV_FILE,header=True)
        print("Loaded CSV File " + CSV_FILE)
        return df

def preprocess(spark_session, df): 
    df = df.filter("lang == 'en'")
    #df = df.rdd.filter(lambda x : len(json.loads(x['media'])['media']) > 0).toDF(sampleRatio = 0.2)
    print("Kept only English Tweets with Media.")
    return df


def save_data(spark_session, df):
    print("Started Saving Users " + str(datetime.datetime.now()))
    __save_users__(df) 
    print("Started Saving Tweets " + str(datetime.datetime.now()))
    __save_tweets__(df)
    print("Started Saving Hashtags " + str(datetime.datetime.now()))
    #__save_hashtags__(df)
    print("Started Saving Media " + str(datetime.datetime.now()))
    #__save_media__(df)
