from pyspark.sql import SparkSession
import data_processor
import image_processor
import learning
import object_detection
import datetime
import multiprocessing as mp


def init_spark():
    spark = SparkSession \
        .builder \
        .appName("Twitter Image Analysis") \
        .getOrCreate()
    print("Spark Initialized!!")
    return spark

def save_twitter_data():
    spark_session = init_spark()
    print("Started Loading File " + str(datetime.datetime.now()))
    df = data_processor.load_file(spark_session)
    print("Started Preprocessing " + str(datetime.datetime.now()))
    #print("Shape of DF loaded : " + str(df.count()))
    df = data_processor.preprocess(spark_session, df)
    print("Started saving to DB " + str(datetime.datetime.now()))
    #print("Shape of DF after filtering non English Tweets : " + str(df.count()))
    data_processor.save_data(spark_session, df)


def download_twitter_media():
    n = 10 #number of processes
    image_tuples = image_processor.get_image_urls_from_db()
    print("Total Images to download " + str(len(image_tuples)))
    image_processor.download_and_resize_images(image_tuples)


def download_twitter_media_mp():
    n = 10 #number of processes
    image_tuples = image_processor.get_image_urls_from_db()
    print("Total Images to download " + str(len(image_tuples)))
    total_length = len(image_tuples)
    chunk_size = round(total_length/n)
    print("chunk size is "  + str(chunk_size))
    tuple_chunks = [image_tuples[i * chunk_size:(i + 1) * chunk_size] for i in range((len(image_tuples) + chunk_size - 1) // chunk_size )]

    processes = []
    for i in range(n):
        print("Starting process " + str(i))
        t = mp.Process(target=image_processor.download_and_resize_images, args=(i, tuple_chunks[i],))
        processes.append(t)
        t.start()
    print(processes)
    for p in processes:
        p.join()

    print("Done")
    #processes = [mp.Process(target=compute, args=(x, 5)) for x in range(8)]
    #image_processor.download_and_resize_images(image_tuples)

def save_image_features():
    images = image_processor.get_image_urls_from_db()
    db_output = []
    for i in images:
        image_path = image_processor.get_full_image_path(i[1])
        result = object_detection.detect_objects_from_images(image_path)
        features_to_save = [i[1]] + result
        db_output.append(features_to_save)
    data_processor.save_image_features(db_output)
    
def main():
    print("Started Pipeline")
    #Read CSV File - spark jobs save the data to tables.
    #Loop through images and download them all.
    #Feed the images to the learning model.
    #Save the clusters to the DB.
    #Combine images for each cluster.
    
    save_twitter_data()
    #download_twitter_media()
    #learning.cluster_images()
    #save_image_features()
    
 

if __name__ == '__main__':
    main()